@@ -55,6 +55,12 @@ TypeCode::TypeCode(TypeCategory f, int kind) {
     break;
   case TypeCategory::Complex:
     switch (kind) {
+    case 2:
+      raw_ = CFI_type_half_float_Complex;
+      break;
+    case 3:
+      raw_ = CFI_type_bfloat_Complex;
+      break;
     case 4:
       raw_ = CFI_type_float_Complex;
       break;
@@ -62,6 +68,8 @@ TypeCode::TypeCode(TypeCategory f, int kind) {
       raw_ = CFI_type_double_Complex;
       break;
     case 10:
+      raw_ = CFI_type_extended_double_Complex;
+      break;
     case 16:
       raw_ = CFI_type_long_double_Complex;
       break;
@@ -126,11 +134,7 @@ TypeCode::GetCategoryAndKind() const {
   case CFI_type_extended_double:
     return std::make_pair(TypeCategory::Real, 10);
   case CFI_type_long_double:
-#if __x86_64__
-    return std::make_pair(TypeCategory::Real, 10);
-#else
     return std::make_pair(TypeCategory::Real, 16);
-#endif
   case CFI_type_float128:
     return std::make_pair(TypeCategory::Real, 16);
   case CFI_type_half_float_Complex:
@@ -144,11 +148,7 @@ TypeCode::GetCategoryAndKind() const {
   case CFI_type_extended_double_Complex:
     return std::make_pair(TypeCategory::Complex, 10);
   case CFI_type_long_double_Complex:
-#if __x86_64__
-    return std::make_pair(TypeCategory::Complex, 10);
-#else
     return std::make_pair(TypeCategory::Complex, 16);
-#endif
   case CFI_type_float128_Complex:
     return std::make_pair(TypeCategory::Complex, 16);
   case CFI_type_char:
