@@ -7,6 +7,12 @@ subroutine s1
   type(t1) :: t1x
   !ERROR: Default-initialized 't1x' must not be initialized in a DATA statement
   data t1x%j / 777 /
+  type :: t2
+    integer, allocatable :: j
+    integer :: k
+  end type t2
+  type(t2) :: t2x
+  data t2x%k / 777 / ! allocatable component is ok
   integer :: ja = 888
   !ERROR: Default-initialized 'ja' must not be initialized in a DATA statement
   data ja / 999 /
@@ -22,6 +28,9 @@ subroutine s1
   integer :: a4(3)
   !ERROR: DATA statement designator 'a4(5_8)' is out of range
   data (a4(j),j=1,5,2) /3*222/
+  integer :: a5(3)
+  !ERROR: DATA statement designator 'a5(-2_8)' is out of range
+  data       a5(-2) / 1 /
   interface
     real function rfunc(x)
       real, intent(in) :: x
