@@ -7,7 +7,6 @@
 //===----------------------------------------------------------------------===//
 
 // UNSUPPORTED: c++03, c++11, c++14, c++17
-// UNSUPPORTED: libcpp-no-concepts
 
 // unique_ptr
 
@@ -20,6 +19,8 @@ static_assert(std::indirectly_writable<std::unique_ptr<int>, int>);
 static_assert(!std::weakly_incrementable<std::unique_ptr<int> >);
 static_assert(std::indirectly_movable<std::unique_ptr<int>, std::unique_ptr<int>>);
 static_assert(std::indirectly_movable_storable<std::unique_ptr<int>, std::unique_ptr<int>>);
+static_assert(std::indirectly_copyable<std::unique_ptr<int>, std::unique_ptr<int>>);
+static_assert(std::indirectly_copyable_storable<std::unique_ptr<int>, std::unique_ptr<int>>);
 static_assert(std::indirectly_swappable<std::unique_ptr<int>, std::unique_ptr<int> >);
 
 static_assert(!std::indirectly_readable<std::unique_ptr<void> >);
@@ -27,3 +28,5 @@ static_assert(!std::indirectly_writable<std::unique_ptr<void>, void>);
 static_assert(!std::weakly_incrementable<std::unique_ptr<void> >);
 static_assert(!std::indirectly_movable<std::unique_ptr<void>, std::unique_ptr<void>>);
 static_assert(!std::indirectly_movable_storable<std::unique_ptr<void>, std::unique_ptr<void>>);
+static_assert(!std::indirectly_copyable<std::unique_ptr<void>, std::unique_ptr<void>>);
+static_assert(!std::indirectly_copyable_storable<std::unique_ptr<void>, std::unique_ptr<void>>);
